'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateArrayPatternConditional = generateArrayPatternConditional;
exports.generateObjectPatternConditional = generateObjectPatternConditional;

var _babelInstance = require('./babelInstance');

var _logical = require('./logical');

var _helpers = require('./helpers');

// key in ref
function generateObjectHasPropertyTest(key, ref) {
  var t = (0, _babelInstance.getTypes)();

  return t.binaryExpression('in', t.stringLiteral(key.name), ref);
} // Generate a LogicalExpression that asserts a ref is compatible with a
// destructuring pattern.

function generateArrayPatternConditional(matchPath, arrayPattern, ref) {
  var t = (0, _babelInstance.getTypes)();
  var elements = arrayPattern.elements;

  var minLength = 0;
  var subconditions = function () {
    var _arr = [];

    for (var _i2 = 0, _len3 = elements.length; _i2 < _len3; _i2++) {
      var _element = elements[_i2];

      var elementType = _element == null ? null : _element.type;

      // Check for mandatory elements
      if (elementType !== "RestElement" && elementType !== "AssignmentPattern") minLength = _i2 + 1;

      if (elementType === "ArrayPattern") {
        _arr.push(generateArrayPatternConditional(matchPath, _element, t.memberExpression(ref, t.numericLiteral(_i2), true)));
      } else if (elementType === "ObjectPattern") {
        _arr.push(generateObjectPatternConditional(matchPath, _element, t.memberExpression(ref, t.numericLiteral(_i2), true)));
      }
    }

    return _arr;
  }();

  var maxLength = !(elements.length > 0 && elements[elements.length - 1].type === "RestElement") ? elements.length : null;

  // Length conditionals
  if (maxLength) {
    subconditions.unshift(t.callExpression((0, _helpers.helper)('hasLength'), [ref, t.numericLiteral(minLength), t.numericLiteral(maxLength)]));
  } else if (minLength > 0) {
    subconditions.unshift(t.callExpression((0, _helpers.helper)('hasLength'), [ref, t.numericLiteral(minLength)]));
  } else {
    subconditions.unshift(t.callExpression((0, _helpers.helper)('hasLength'), [ref]));
  }return (0, _logical.createAndExpression)(subconditions);
}

function generateObjectPatternConditional(matchPath, objectPattern, ref) {
  var t = (0, _babelInstance.getTypes)();
  var properties = objectPattern.properties;
  var subconditions = [t.callExpression((0, _helpers.helper)('isObject'), [ref])];

  for (var _i3 = 0, _len4 = properties.length; _i3 < _len4; _i3++) {
    var _ref;

    var _property = properties[_i3];

    var key = _property.key;
    var valueType = (_ref = _property.value) == null ? null : _ref.type;

    if (valueType !== "AssignmentPattern") {
      subconditions.push(generateObjectHasPropertyTest(key, ref));
    }
    if (valueType === "ArrayPattern") {
      subconditions.push(generateArrayPatternConditional(matchPath, _property.value, t.memberExpression(ref, key)));
    } else if (valueType === "ObjectPattern") {
      subconditions.push(generateObjectPatternConditional(matchPath, _property.value, t.memberExpression(ref, key)));
    }
  }

  return (0, _logical.createAndExpression)(subconditions);
}