'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformMatchExpression = transformMatchExpression;
exports.transformMatchStatement = transformMatchStatement;

var _babelInstance = require('./babelInstance');

var _blocks = require('./blocks');

var _is = require('./is');

var _ref4 = require('./ref');

var _destructure = require('./destructure');

var _logical = require('./logical');

var _getLoc = require('ast-loc-utils/lib/getLoc');

var _getLoc2 = _interopRequireDefault(_getLoc);

var _buildAtLoc = require('ast-loc-utils/lib/buildAtLoc');

var _buildAtLoc2 = _interopRequireDefault(_buildAtLoc);

var _placeAtLoc = require('ast-loc-utils/lib/placeAtLoc');

var _placeAtLoc2 = _interopRequireDefault(_placeAtLoc);

var _span = require('ast-loc-utils/lib/span');

var _span2 = _interopRequireDefault(_span);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function transformMatchPlaceholders(path, discriminantRef) {
  return path.traverse({
    noScope: true // linter compatibility

    , PlaceholderExpression(phPath) {
      return phPath.replaceWith(discriminantRef);
    } });
}

function createMatchTest(casePath, matchTransformationState) {
  var _ref2;

  var matchPath = matchTransformationState.matchPath,
      discriminantRef = matchTransformationState.discriminantRef;

  var exprs = [];

  var caseNode = casePath.node;
  var testPath = casePath.get("test");
  var testNode = testPath.node;
  if (testNode) {
    exprs.push(createMatchUnboundTest(casePath, testPath, testNode, matchTransformationState));
  }var _ref = (_ref2 = caseNode.binding) == null ? null : _ref2.type;
  if (_ref === "ArrayPattern") {
    exprs.push((0, _destructure.generateArrayPatternConditional)(matchPath, caseNode.binding, discriminantRef));
  } else if (_ref === "ObjectPattern") {
    exprs.push((0, _destructure.generateObjectPatternConditional)(matchPath, caseNode.binding, discriminantRef));
  }return (0, _logical.createAndExpression)(exprs);
}

function isUndefined(node) {
  return node.type === "Identifier" && node.name === "undefined";
}

function isSignedNumber(node) {
  return node.type === "UnaryExpression" && (node.operator === "+" || node.operator === "-") && node.argument.type === "NumericLiteral";
}

function createMatchTestReplacement(node, discriminantRef) {
  if ((0, _is.isa)(node, "LogicalExpression")) {
    node.left = createMatchTestReplacement(node.left, discriminantRef);
    node.right = createMatchTestReplacement(node.right, discriminantRef);
    return node;
  } else if ((0, _is.isa)(node, "UnaryExpression") && node.operator === "!") {
    node.argument = createMatchTestReplacement(node.argument, discriminantRef);
    return node;
  } else if ((0, _is.isa)(node, "RegExpLiteral")) {
    var t = (0, _babelInstance.getTypes)();
    var loc = (0, _getLoc2.default)(node);
    return (0, _buildAtLoc2.default)(loc, t.callExpression, (0, _buildAtLoc2.default)(loc, t.memberExpression, node, (0, _buildAtLoc2.default)(loc, t.identifier, "test")), [discriminantRef]);
  } else if (isUndefined(node) || (0, _is.isa)(node, "Literal") || isSignedNumber(node)) {
    var _t = (0, _babelInstance.getTypes)();
    var _loc = (0, _getLoc2.default)(node);
    return (0, _buildAtLoc2.default)(_loc, _t.binaryExpression, "===", discriminantRef, node);
  } else {
    return node;
  }
}

function createMatchUnboundTest(casePath, testPath, testNode, matchTransformationState) {
  var discriminantRef = matchTransformationState.discriminantRef;


  testPath.replaceWith(createMatchTestReplacement(testNode, discriminantRef));
  transformMatchPlaceholders(testPath, discriminantRef);

  return testPath.node;
}

function bindingToAnonymousBlock(binding, body, discriminantRef) {
  var t = (0, _babelInstance.getTypes)();
  var newBody = (0, _blocks.toBlockStatement)(body);
  // const <binding> = _ref
  var loc = (0, _getLoc2.default)(binding);
  newBody.body.unshift((0, _buildAtLoc2.default)(loc, t.variableDeclaration, "const", [(0, _buildAtLoc2.default)(loc, t.variableDeclarator, binding, discriminantRef)]));
  return newBody;
}

function createMatchConsequent(casePath, discriminantRef) {
  var caseNode = casePath.node;

  if (caseNode.binding) {
    var binding = caseNode.binding,
        consequent = caseNode.consequent;

    return bindingToAnonymousBlock(binding, consequent, discriminantRef);
  } else {
    var _consequent = caseNode.consequent;

    return (0, _blocks.toStatement)(_consequent);
  }
}

function createMatchContinuation(idx, matchTransformationState) {
  var _ref3;

  var t = (0, _babelInstance.getTypes)();
  var matchPath = matchTransformationState.matchPath,
      cases = matchTransformationState.cases,
      discriminantRef = matchTransformationState.discriminantRef;


  if (idx >= cases.length) {
    // Don't make empty else clauses.
    return idx > 0 ? null : (0, _buildAtLoc2.default)((0, _span2.default)((0, _getLoc2.default)(matchPath.node), -1), t.emptyStatement);
  }var c = cases[idx];
  var casePath = matchPath.get(`cases.${idx}`);

  var consequent = createMatchConsequent(casePath, discriminantRef);
  if (((_ref3 = c.test) == null ? null : _ref3.type) === "MatchElse") return consequent;
  var test = createMatchTest(casePath, matchTransformationState);
  var continuation = createMatchContinuation(idx + 1, matchTransformationState);

  return (0, _buildAtLoc2.default)((0, _getLoc2.default)(casePath.node), t.ifStatement, test, consequent, continuation);
}

function transformMatchExpression(path) {
  var t = (0, _babelInstance.getTypes)();

  var node = path.node;
  var originalDiscriminant = node.discriminant;
  node.type = "MatchStatement";

  var argRef = path.scope.generateUidIdentifier("it");
  (0, _placeAtLoc2.default)(argRef, (0, _span2.default)((0, _getLoc2.default)(node), 1));
  node.discriminant = argRef;

  var surroundingLoc = (0, _getLoc2.default)(node);

  // Transform to IIFE wrapping MatchStatement.
  return path.replaceWith((0, _buildAtLoc2.default)(surroundingLoc, t.callExpression, (0, _buildAtLoc2.default)(surroundingLoc, t.arrowFunctionExpression, [argRef], (0, _buildAtLoc2.default)(surroundingLoc, t.blockStatement, [node])), [originalDiscriminant]));
}

function transformMatchStatement(path) {
  var t = (0, _babelInstance.getTypes)();
  var node = path.node;

  var _undeclaredRef = (0, _ref4.undeclaredRef)(path, node.discriminant, "it"),
      ref = _undeclaredRef.ref,
      declarator = _undeclaredRef.declarator,
      isComplex = _undeclaredRef.isComplex;

  if (isComplex) {
    path.insertBefore((0, _buildAtLoc2.default)((0, _getLoc2.default)(declarator), t.variableDeclaration, "const", [declarator]));
  }var matchTransformationState = {
    matchPath: path,
    discriminantRef: ref,
    cases: node.cases
  };

  return path.replaceWith(createMatchContinuation(0, matchTransformationState));
}